<div class="panel panel-default">
    <div class="panel-heading">Agent Status
    </div>
    <div class="panel-body">

        <table class="table">
            <thead>

            <tr>
                <th></th>
                <th>Is Running</th>
                <th>Queue Size</th>
                <th>Total Processed</th>
                <th>Last Error</th>
            </tr>

            </thead>

            <tbody>

            <!-- LiveDB Value Updater -->
            <tr>
                <th>Value Updater</th>
                <td>{{item.rawValueQueueStatus}}</td>
                <td>{{item.rawValueQueueSize}}</td>
                <td>{{item.rawValueProcessedTotal}}</td>
                <td>{{item.rawValueLastError}}</td>

            </tr>

            </tbody>
        </table>
    </div>
</div>